# Battlefield 2142 -- example player connect checking system module.
#
# Default module for handling how players can be denied access
# to a server.
#
# Copyright (c)2006 Digital Illusions CE AB
# Author: Kristoffer Benjaminsson

import bf2
import host
from bf2 import g_debug

# Init. Registers the necessary handlers
def init():
	if g_debug: print 'initializing player connect checking script'
	
	host.registerHandler('PlayerChallenge', onPlayerChallenge, 1)
	host.registerHandler('PrePlayerConnect', onPrePlayerConnect, 1)
	host.registerHandler('ValidatePlayerNameResponse', onPlayerNameValidated, 1) #Added by Niklas Henriks to validate player Name and ID for update 1.50.




# onPlayerChallenge is called by the engine as part of the
# challenge-response handshaking. At this time only client ID and rank
# is available. Servers can use this information to deny access
# based on the players rank.
# Only called for ranked servers
def onPlayerChallenge(clientId, rank):
	if g_debug: print "onPlayerChallenge: " + ":" + str(clientId) + ":" + str(rank)
	
	# this can happen if it's the first time a player runs
	# the game on a computer and no stats were retrieved
	if rank == -1:
		return
		
	if bf2.serverSettings.getMaxRank() > 0 and rank > bf2.serverSettings.getMaxRank():
		bf2.gameServer.abortCurrentConnectionAttempt(clientId)




# onPrePlayerConnect is called by the engine after the challenge-response
# handshake is done, but prior to the player actually beeing created.
# The clientId passed is the same as passed to the onPlayerChallenge event.
def onPrePlayerConnect(clientId, playerNick):
	if g_debug: print "onPrePlayerConnect: " + playerNick + ":" + str(clientId)

#	if playerNick == "some_nick_to_deny": bf2.gameServer.abortCurrentConnectionAttempt(clientId)



# onPlayerNameValidated is called by the engine when a reply from GameSpy arives, this is done directly when the player connects to the game server.
def onPlayerNameValidated(realNick, oldNick, realPID, oldPID, player):
	if realNick != oldNick:
		message = "Player: " + oldNick + " -real name: " + realNick + " was kicked and banned for using Name Hacks"
		host.rcon_invoke("admin.banPlayer " + str(player.index) + " 604800")
		host.sgl_sendTextMessage(0, 12, 1, message, 0)
		return
	if realPID != oldPID:
		message = "Player: " + oldNick + " tried to use PlayerID:" + str(oldPID) + " when s/he's real PlayerID is " + str(realPID) + " and was kicked and banned for using PID Hacks"
		host.rcon_invoke("admin.banPlayer " + str(player.index) + " 604800")
		host.sgl_sendTextMessage(0, 12, 1, message, 0)
