var fs = require('fs'),
    path = require('path'),
    async = require('async'),
    config = require('./config.json');

var log = require('winston');

// get our game args
var args = process.argv.slice(2);
if (!args[0]) {
    log.error('SNAPSHOT-UPLOADER: no game defined in argument 1!');
}

var game = args[0];
var dir = config.snapshots.games[game].pendingDir;
var dirSent = config.snapshots.games[game].successDir;
var dirError = config.snapshots.games[game].errorDir;
if (!fs.existsSync(dir)) {
    console.error('SNAPSHOT-UPLOADER [' + game + ']: exiting! queue directory ' + dir + ' not found!')
    process.exit();
}


log.info('SNAPSHOT-UPLOADER [' + game + ']: starting');

// AWS Stuff (should have a better way sooner or later, so we don't need the whole aws-sdk on gnome.)
var AWS = require('aws-sdk');

AWS.config.loadFromPath(path.join(config.root, 'aws.json'));

var s3bucket = new AWS.S3({ params: { Bucket: config.snapshots.games[game].s3bucket } });

function doWork() {
        var files = fs.readdirSync(dir);
        files.sort(function(a, b) {
                return fs.statSync(path.join(dir, a)).mtime.getTime() - fs.statSync(path.join(dir, b)).mtime.getTime();
        });

        if (files.length > 0) {

        } else {
                return setTimeout(doWork, config.snapshots.games[game].scanInterval);
        }

        async.eachSeries(files, function(filename, done) {
                var body = fs.readFileSync(path.join(dir, filename));
                log.info('SNAPSHOT-UPLOADER [' + game + ']: read ' + body.length + ' bytes from ' + filename);

                s3bucket.upload({Key: filename, Body: body}, function(err, data) {
                        if (err) {
                                log.info('SNAPSHOT-UPLOADER [' + game + ']: error uploading ' + filename)
                                log.error(err);
                                fs.rename(path.join(dir, filename), path.join(dirError, filename));
                        } else {
                                if (config.snapshots.games[game].deleteOnSuccess) {
                                    fs.unlink(path.join(dir, filename));
                                } else {
                                    fs.rename(path.join(dir, filename), path.join(dirSent, filename));
                                }
                                log.info('SNAPSHOT-UPLOADER [' + game + ']: success ' + filename);
                        }
                        setTimeout(done, config.snapshots.games[game].uploadInterval);
                });

        }, function(err) {
                setTimeout(doWork, config.snapshots.games[game].scanInterval);
        })
}

doWork();
